local bs = AceLibrary("Babble-Spell-2.2")
local AceAddon = AceLibrary("AceAddon-2.0")

local L = AceLibrary("AceLocale-2.2"):new("Cartographer_Mining")
L:RegisterTranslations("enUS", function() return {
    ["^You perform %s on (.+)%%.$"] = true,

    ["Vein"] = true,
    ["Deposit"] = true,

    ["Copper"] = true,
    ["Tin"] = true,
    ["Iron"] = true,
    ["Silver"] = true,
    ["Gold"] = true,
    ["Mithril"] = true,
    ["Truesilver"] = true,
    ["Thorium"] = true,
    ["Small Thorium"] = true,
    ["Rich Thorium"] = true,
    ["Dark Iron"] = true,
} end)

local mod = Cartographer:NewModule(bs["Mining"], "AceConsole-2.0", "AceEvent-2.0")

mod.icon = {
    ["Copper"] = {
        text = L["Copper"],
        path = "Interface\\Icons\\INV_Ore_Copper_01",
        width = 12,
        height = 12
    },
    ["Tin"] = {
        text = L["Tin"],
        path = "Interface\\Icons\\INV_Ore_Tin_01",
        width = 12,
        height = 12
    },
    ["Silver"] = {
        text = L["Silver"],
        path = "Interface\\Icons\\INV_Stone_16",
        width = 12,
        height = 12
    },
    ["Iron"] = {
        text = L["Iron"],
        path = "Interface\\Icons\\INV_Ore_Iron_01",
        width = 12,
        height = 12
    },
    ["Gold"] = {
        text = L["Gold"],
        path = "Interface\\Icons\\INV_Ore_Copper_01",
        width = 12,
        height = 12
    },
    ["Mithril"] = {
        text = L["Mithril"],
        path = "Interface\\Icons\\INV_Ore_Mithril_02",
        width = 12,
        height = 12
    },
    ["Truesilver"] = {
        text = L["Truesilver"],
        path = "Interface\\Icons\\INV_Ore_TrueSilver_01",
        width = 12,
        height = 12
    },
    ["Small Thorium"] = {
        text = L["Small Thorium"],
        path = "Interface\\Icons\\INV_Ore_Thorium_02",
        width = 12,
        height = 12
    },
    ["Thorium"] = {
        text = L["Thorium"],
        path = "Interface\\Icons\\INV_Ore_Thorium_02",
        width = 12,
        height = 12
    },
    ["Rich Thorium"] = {
        text = L["Rich Thorium"],
        path = "Interface\\Icons\\INV_Ore_Thorium_02",
        width = 12,
        height = 12
    },
    ["Dark Iron"] = {
        text = L["Dark Iron"],
        path = "Interface\\Icons\\INV_Ore_Mithril_01",
        width = 12,
        height = 12
    },
}

local lua51
function mod:OnInitialize()
    local aceopts = AceAddon:GetAceOptionsDataTable(self)
    aceopts.toggle = {
        name  = AceLibrary('AceLocale-2.2'):new('Cartographer')['Toggle'],
        desc  = AceLibrary('AceLocale-2.2'):new('Cartographer')['Toggle the module on and off.'],
        type  = 'toggle',
        order = -1,
        get   = function() return Cartographer:IsModuleActive(self) end,
        set   = function() Cartographer:ToggleModuleActive(self) end,
    }

    Cartographer.options.args[bs["Mining"]] = {
        name = bs["Mining"],
        desc = self.notes,
        type = 'group',
        args = aceopts,
        handler = self,
    }
    Cartographer:GetModule('Professions').addons[bs["Mining"]] = self

    if not Cartographer_MiningDB then
        Cartographer_MiningDB = {}
    else
		for _, zone in pairs(Cartographer_MiningDB) do
			for id, data in pairs(zone) do
				if type(data) == "table" then
					zone[id] = data.icon
				end
			end
		end
    end

    lua51 = loadstring("return function(...) return ... end") and true or false
end

function mod:OnEnable()
    if Cartographer_Notes then
        if not self.iconsregistered then
            for k,v in pairs(self.icon) do
                Cartographer_Notes:RegisterIcon(k, v)
            end
            self.iconsregistered = true
        end

        Cartographer_Notes:RegisterNotesDatabase('Mining', Cartographer_MiningDB, self)
    else
        Cartographer:ToggleModuleActive(self, false)
    end

    self:RegisterEvent("UI_ERROR_MESSAGE")

    if lua51 then
        self:RegisterEvent("CHAT_MSG_SPELL_SELF_BUFF")
    else
        self:RegisterEvent("SPELLCAST_START")
    end
end

function mod:OnDisable()
    self:UnregisterAllEvents()

    if Cartographer_Notes then
        Cartographer_Notes:UnregisterNotesDatabase('Mining')
    end
end

function mod:SetNote(what)
    local x, y = GetPlayerMapPosition("player")
    if x == 0 and y == 0 then return end
    local zone = GetRealZoneText()
	Cartographer_Notes:SetCustomNote(zone, x, y, L:GetReverseTranslation(what), "Mining")
--    Cartographer_Notes:SetNote(zone, x, y, what, 0, 0.8, 0, "", 1, 1, 1, "", 1, 1, 1, "Mining", L:GetReverseTranslation(what))
end

function mod:OnNoteTooltipRequest(zone, id)
	local icon = Cartographer_MiningDB[zone][id]
	
	Tablet:SetTitle(L[icon])
	Tablet:SetTitleColor(0, 0.8, 0)
	
	Tablet:AddCategory(
		'columns', 2,
		'hideBlankLine', true
	):AddLine(
		'text', AceLibrary("AceLocale-2.2"):new("Cartographer-Notes")["Created by"],
		'text2', "Mining"
	)
end

function mod:RegisterOreFromTooltip()
    local what = GameTooltipTextLeft1:GetText()
    if what and strlen(what) > 0 then
        if string.find(what, L["Dark Iron"]) then
            self:SetNote(L["Dark Iron"])
        elseif string.find(what, L["Rich Thorium"]) then
            self:SetNote(L["Dark Thorium"])
        elseif string.find(what, L["Small Thorium"] ) then
            self:SetNote(L["Small Thorium"])
        else
            what = string.gsub(what, " %(%d+%)", "")
            local _, _, ore, veindep = string.find(what, "([^ ]+) ([^ ]+)$")
            if ore and veindep and veindep == L["Vein"] or veindep == L["Deposit"] then
                self:SetNote(ore)
            end
        end
    end
end

function mod:CHAT_MSG_SPELL_SELF_BUFF(msg)
    local _,_,what = string.find(msg, string.format(L["^You perform %s on (.+)%%.$"], bs["Mining"]))
    if what then
        self:RegisterOreFromTooltip()
    end
end

function mod:UI_ERROR_MESSAGE(msg)
    if string.find(msg, UNIT_SKINNABLE_ROCK) then
        local what = GameTooltipTextLeft1:GetText()
        if what and strlen(what) > 0 then
            self:RegisterOreFromTooltip()
        end
    end
end

function mod:SPELLCAST_START(msg)
    if msg == bs["Mining"] then
        local what = GameTooltipTextLeft1:GetText()
        if what and strlen(what) > 0 then
            self:RegisterOreFromTooltip()
        end
    end
end
